@@ -46,9 +46,17 @@ META_DF = pd.DataFrame(
 df_empty = pd.DataFrame([], columns=IAMC_IDX + [2005, 2010])
 
 
-def test_init_df_with_index(test_pd_df):
-    df = IamDataFrame(test_pd_df.set_index(META_IDX))
-    pd.testing.assert_frame_equal(df.timeseries().reset_index(), test_pd_df)
+@pytest.mark.parametrize("index", (None, META_IDX, ["model"]))
+def test_init_df(test_pd_df, index):
+    """Casting to IamDataFrame and returning as `timeseries()` yields original frame"""
+
+    # set a value to `nan` to check that timeseries columns are ordered correctly
+    test_pd_df.loc[0, test_pd_df.columns[5]] = np.nan
+
+    # any number of columns can be set as index
+    df = test_pd_df.copy() if index is None else test_pd_df.set_index(index)
+    obs = IamDataFrame(df).timeseries()
+    pdt.assert_frame_equal(obs, test_pd_df.set_index(IAMC_IDX), check_column_type=False)
 
 
 def test_init_from_iamdf(test_df_year):
@@ -94,7 +102,6 @@ def test_init_df_with_illegal_values_raises(test_pd_df, illegal_value):
         f'.*string "{illegal_value}" in `data`:'
         r"(\n.*){2}model_a.*scen_a.*World.*Primary Energy.*EJ/yr.*2005"
     )
-
     with pytest.raises(ValueError, match=msg):
         IamDataFrame(test_pd_df)
 
@@ -102,7 +109,12 @@ def test_init_df_with_illegal_values_raises(test_pd_df, illegal_value):
 def test_init_df_with_na_scenario(test_pd_df):
     # missing values in an index dimension raises an error
     test_pd_df.loc[1, "scenario"] = np.nan
-    pytest.raises(ValueError, IamDataFrame, data=test_pd_df)
+    msg = (
+        "Empty cells in `data` \(columns: 'scenario'\):"
+        r"(\n.*){2}model_a.*NaN.*World.*Primary Energy|Coal.*EJ/yr.*2005.*"
+    )
+    with pytest.raises(ValueError, match=msg):
+        IamDataFrame(test_pd_df)
 
 
 def test_init_df_with_float_cols(test_pd_df):
@@ -177,18 +189,28 @@ def test_init_empty_message(caplog):
     assert caplog.records[message_idx].levelno == logging.WARNING
 
 
-def test_init_with_column_conflict(test_pd_df):
-    # add a column to the timeseries data with a conflict to the meta attribute
-    test_pd_df["meta"] = "foo"
+def test_init_with_unnamed_column(test_pd_df):
+    # add a column to the timeseries data with an unnamed column
+    test_pd_df[None] = "foo"
 
-    # check that initialising an instance with an extra-column `meta` raises
-    msg = re.compile(r"Column name \['meta'\] is illegal for timeseries data.")
+    # check that initialising an instance with an unnamed column raises
+    with pytest.raises(ValueError, match="Unnamed column in `data`: None"):
+        IamDataFrame(test_pd_df)
+
+
+@pytest.mark.parametrize("illegal", ["meta", ""])
+def test_init_with_illegal_column(test_pd_df, illegal):
+    # add a column to the timeseries data with an illegal column name
+    test_pd_df[illegal] = "foo"
+
+    # check that initialising an instance with an illegal column name raises
+    msg = f"Illegal column for timeseries data: '{illegal}'"
     with pytest.raises(ValueError, match=msg):
         IamDataFrame(test_pd_df)
 
     # check that recommended fix works
-    df = IamDataFrame(test_pd_df, meta_1="meta")
-    assert df.meta_1 == ["foo"]
+    df = IamDataFrame(test_pd_df, valid=illegal)
+    assert df.valid == ["foo"]
 
 
 def test_set_meta_with_column_conflict(test_df_year):
